unit Graf3DPrimitives;
interface

 uses
  FixMath, Graf3D, Graf3DScene;

 const
  CIR_VERTICES = 12;

{parts}
 procedure AddTri3D (var shape: Shape3D; x, y, z, w, h: Longint);
 procedure AddRec3D (var shape: Shape3D; x, y, z, w, h: Longint);
 procedure AddCir3D (var shape: Shape3D; x, y, z, d: Longint);
{2d}
 procedure SetTri3D (var shape: Shape3D; x, y, z, w, h: Longint);
 procedure SetRec3D (var shape: Shape3D; x, y, z, w, d: Longint);
 procedure SetCir3D (var shape: Shape3D; x, y, z, r: Longint);
{3d}
 procedure SetWed3D (var shape: Shape3D; x, y, z, w, h, d: Longint);
 procedure SetBox3D (var shape: Shape3D; x, y, z, w, h, d: Longint);
 procedure SetCyl3D (var shape: Shape3D; x, y, z, w, h, d, r: Longint);

implementation

 procedure AddTri3D (var shape: Shape3D; x, y, z, w, h: Longint);
 begin
  AddVertice3D(shape, x, y + h div 2, z);
  AddVertice3D(shape, x + w div 2, y - h div 2, z);
  AddVertice3D(shape, x - w div 2, y - h div 2, z);
  AddEdge3D(shape, shape.verticesLength, shape.verticesLength - 1);
  AddEdge3D(shape, shape.verticesLength - 1, shape.verticesLength - 2);
  AddEdge3D(shape, shape.verticesLength - 2, shape.verticesLength);
  AddFace3D(shape, shape.verticesLength - 2, shape.verticesLength - 1, shape.verticesLength);
 end;

 procedure AddRec3D (var shape: Shape3D; x, y, z, w, h: Longint);
 begin
  AddVertice3D(shape, x + w div 2, y + h div 2, z);
  AddVertice3D(shape, x + w div 2, y - h div 2, z);
  AddVertice3D(shape, x - w div 2, y - h div 2, z);
  AddVertice3D(shape, x - w div 2, y + h div 2, z);
  AddEdge3D(shape, shape.verticesLength, shape.verticesLength - 1);
  AddEdge3D(shape, shape.verticesLength - 1, shape.verticesLength - 2);
  AddEdge3D(shape, shape.verticesLength - 2, shape.verticesLength - 3);
  AddEdge3D(shape, shape.verticesLength - 3, shape.verticesLength);
  AddFace3D(shape, shape.verticesLength - 2, shape.verticesLength - 1, shape.verticesLength);
  AddFace3D(shape, shape.verticesLength - 3, shape.verticesLength - 2, shape.verticesLength);
 end;

 procedure AddCir3D (var shape: Shape3D; x, y, z, d: Longint);
  var
   i: Integer;
   radians, arc: Fixed;
 begin
  arc := FracDiv(360, Long2Fix(CIR_VERTICES));
  for i := 1 to CIR_VERTICES do
   begin
    radians := FracMul(FixMul(arc, Long2Fix(i)), FixDiv(205887, 180));
    AddVertice3D(shape, Fix2Long(Long2Fix(x) + FracMul(Long2Fix(d div 2), FracCos(radians))), Fix2Long(Long2Fix(y) + FracMul(Long2Fix(d div 2), FracSin(radians))), z);
   end;
  for i := 1 to CIR_VERTICES - 1 do
   begin
    AddEdge3D(shape, shape.verticesLength - CIR_VERTICES + i, shape.verticesLength - CIR_VERTICES + i + 1);
   end;
  AddEdge3D(shape, shape.verticesLength - CIR_VERTICES + 1, shape.verticesLength);
 end;

{publics}
 procedure SetTri3D (var shape: Shape3D; x, y, z, w, h: Longint);
 begin
  AddTri3D(shape, x, y, z, w, h);
 end;

 procedure SetRec3D (var shape: Shape3D; x, y, z, w, d: Longint);
 begin
  AddRec3D(shape, x, y, z, w, d);
 end;

 procedure SetCir3D (var shape: Shape3D; x, y, z, r: Longint);
 begin
  AddCir3D(shape, x, y, z, r);
 end;

 procedure SetWed3D (var shape: Shape3D; x, y, z, w, h, d: Longint);
  var
   i: Integer;
 begin
  SetPt3D(shape.origin, Long2Fix(x), Long2Fix(y), Long2Fix(z));
  AddTri3D(shape, x, y, z + d div 2, w, h);
  AddTri3D(shape, x, y, z - d div 2, w, h);
  for i := 1 to 3 do
   AddEdge3D(shape, i, i + 3);
 end;

 procedure SetBox3D (var shape: Shape3D; x, y, z, w, h, d: Longint);
  var
   i: Integer;
 begin
  SetPt3D(shape.origin, Long2Fix(x), Long2Fix(y), Long2Fix(z));
  AddRec3D(shape, x, y, z + d div 2, w, h);
  AddRec3D(shape, x, y, z - d div 2, w, h);
  for i := 1 to 4 do
   AddEdge3D(shape, i, i + 4);
 end;

 procedure SetCyl3D (var shape: Shape3D; x, y, z, w, h, d, r: Longint);
  var
   i: Integer;
 begin
  SetPt3D(shape.origin, Long2Fix(x), Long2Fix(y), Long2Fix(z));
  AddCir3D(shape, x, y, z + d div 2, r);
  AddCir3D(shape, x, y, z - d div 2, r);
  for i := 1 to CIR_VERTICES do
   AddEdge3D(shape, i, i + CIR_VERTICES);
 end;

end.